﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using Microsoft.Phone.Controls;
using Microsoft.Phone.Shell;
using game.Pages;
using game.Model;

namespace game
{
    public partial class MainPage : PhoneApplicationPage
    {
        public SoundPlayer playSound;
        public SoundEffectAction soundEffect;
        private UIElement actualView;
        
        public HomePage homePage;
        public Settings settingsPage;
        public GameSettings gameSettings;
        public PanoramaPage panoramaPage;
        public Intro introPage;
        public Game gamePage;
        public bool navigation = false;

        // Constructor
        public MainPage()
        {
            InitializeComponent();

            readSettings();
            //settingsPage = new Settings(this);
            //body.Children.Add(intro);
            playSound = new SoundPlayer(this);

            soundEffect = new SoundEffectAction();
            soundEffect.SetupSoundEffect("sounds/goodmove.wav");

            introPage = new Intro(this);
            homePage = new HomePage(this);
            gamePage = new Game(this);



            //panoramaPage = new PanoramaPage(this);

            actualView = introPage;
            LayoutRoot.Children.Add(introPage);

        }

        public void readSettings() {
            ReadWrite rw = new ReadWrite();
            gameSettings = new GameSettings(rw.LoadData("settings.txt"));
        }

        public void changeBackgroundBig() {
            ImageSourceConverter c = new ImageSourceConverter();
            backImg.Source = (ImageSource)c.ConvertFromString("Images/gameE.png");
        }

        public void changeBackgroundSmall()
        {
            ImageSourceConverter c = new ImageSourceConverter();
            backImg.Source = (ImageSource)c.ConvertFromString("Images/game.png");
        }

        public void restoreBackground()
        {
            ImageSourceConverter c = new ImageSourceConverter();
            backImg.Source = (ImageSource)c.ConvertFromString("Images/main4.jpg");
        }

        public void goToHome(UIElement previous)
        {
            LayoutRoot.Children.Remove(previous);
            actualView = homePage;
            homePage.init();
            LayoutRoot.Children.Add(homePage);
        }
        public void goToIntro(UIElement previous)
        {
            if (this.gameSettings.getSound() && !playSound.isChildren)
                playSound.AddToContainer();
            LayoutRoot.Children.Remove(previous);
            actualView = introPage;
            LayoutRoot.Children.Add(introPage);
        }
        public void goToGame(UIElement previous)
        {
            if (this.gameSettings.getSound() && !playSound.isChildren)
                playSound.AddToContainer();

            LayoutRoot.Children.Remove(previous);
            gamePage.init();
            actualView = gamePage;
            LayoutRoot.Children.Add(gamePage);
        }

        public void goToSettings(UIElement previous)
        {
            LayoutRoot.Children.Remove(previous);
            actualView = settingsPage;
            LayoutRoot.Children.Add(settingsPage);
        }

        public void goToPanorama(UIElement previous, int defaultValue)
        {
            LayoutRoot.Children.Remove(previous);
            actualView = panoramaPage;
            panoramaPage.refreshHighScores();
            LayoutRoot.Children.Add(panoramaPage);
            panoramaPage.SetDefaultItem(defaultValue);
        }


        protected override void OnBackKeyPress(System.ComponentModel.CancelEventArgs e)
        {

            if (actualView != gamePage || !gamePage.keepCounting)
                return;
            
            base.OnBackKeyPress(e);

            MessageBoxResult result = MessageBox.Show("", "Are you sure you want to leave the application?", MessageBoxButton.OKCancel);

            if ( result == MessageBoxResult.OK )
            {
                //save
                ReadWrite rw = new ReadWrite();
                rw.SaveData(gamePage.currentTime + "," + gamePage.points, "state.txt");
                MessageBox.Show("Current game Saved!");
            }
            else
            {
                //nothing
                //MessageBox.Show("Bye!");
                e.Cancel = true;
            }
        }

        /*
         * this is called when we leave this page
         */
        protected override void OnNavigatedFrom(System.Windows.Navigation.NavigationEventArgs e)
        {
            if (actualView != gamePage || !gamePage.keepCounting)
                return;

            ReadWrite rw = new ReadWrite();
            rw.SaveData(gamePage.currentTime + "," + gamePage.points, "state.txt");
        }

        /*
         * this is called when we arrive to this page
         */
        protected override void OnNavigatedTo(System.Windows.Navigation.NavigationEventArgs e)
        {
            base.OnNavigatedTo(e);
            readSettings();

            
            if (!this.gameSettings.getSound() || navigation)
                playSound.RemoveFromContainer();


            

            navigation = true;
            
        }

        /*
         * this is called when the user press "W" button
         */
        private void Application_Deactivated( object sender, DeactivatedEventArgs e)
        {

            if (actualView != gamePage || !gamePage.keepCounting)
                return;

            ReadWrite rw = new ReadWrite();
            rw.SaveData(gamePage.currentTime + "," + gamePage.points, "state.txt");
        }
    }
}