﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using Microsoft.Phone.Controls;
using System.Threading;
using System.ComponentModel;
using System.Windows.Threading;
using game.Model;


namespace game.Pages
{
    public partial class Game : PhoneApplicationPage, IGame
    {
        GridMonitor monitor;
        MainPage main;
        public int points;
        public int currentTime;
        DispatcherTimer tmr;
        Queue<int> wrongMoves;
        CountDown countDown;
        public bool keepCounting = true;
        Random r;
        

        public Game(MainPage main)
        {
            this.main = main;
            InitializeComponent();
            monitor = new GridMonitor(this, iconToFind);
            backButton.Click += new RoutedEventHandler(backButtonClicked);
        }


        public void init()
        {

            main.readSettings();
            ReadWrite rw = new ReadWrite();
            ActualState actualState = new ActualState(rw.LoadData("state.txt"));
            rw.TruncateData("state.txt");

            body.Children.Clear();
            points = actualState.points;
            keepCounting = true;
            wrongMoves = new Queue<int>();
            score.Text = points+" pts";
            score.Foreground = new SolidColorBrush(Colors.White);
            iconToFind.Foreground = new SolidColorBrush(Colors.White);
            r = new Random(DateTime.Now.Millisecond);
            //monitor actions
            if (main.gameSettings.getHardness())
            {
                main.changeBackgroundSmall();
                monitor.setRowsAndCols(6, 5);
                monitor.setIcons(r.Next(22, 27));
            }
            else
            {
                main.changeBackgroundBig();
                monitor.setRowsAndCols(5, 4);
                monitor.setIcons(r.Next(15, 18));
            }

           
            
            body.Children.Add(monitor.getGrid());
            monitor.toFind();

            //play sound!
            if (main.gameSettings.getSound())
                main.playSound.playSound(new Uri("sounds/Mandinga - Soarele Meu.mp3", UriKind.Relative));

            //timer
            countDown = new CountDown(actualState.time);
            if (tmr == null)
            {
                tmr = new DispatcherTimer();
                tmr.Interval = TimeSpan.FromSeconds(0.001);
                tmr.Tick += OnTimerTick;
                tmr.Start();
            }
        }

        void OnTimerTick(object sender, EventArgs e)
        {
            if (!keepCounting)
                return;

            timer.Foreground = new SolidColorBrush(Colors.White);
            int secsToRemove = 1;
            if (wrongMoves.Count != 0)
            {
                secsToRemove = 500;
                timer.Foreground = new SolidColorBrush(Colors.Red);
                wrongMoves.Dequeue();
            }

            if (!countDown.secLess(secsToRemove))
            {
                //time off
                noMovesLeft();
            }
            currentTime = countDown.sec;
            timer.Text = countDown.print();        
        } 

        void backButtonClicked(object sender, RoutedEventArgs e)
        {
            if (main.gameSettings.getEffectsSound())
            {
                main.soundEffect.SetupSoundEffect("sounds/click.wav");
                main.soundEffect.Play();
            }
            main.restoreBackground();
            main.goToHome(this);
            main.playSound.stopSound();
        }

        public void wrongMove()
        {
            //reduce time!
            if (main.gameSettings.getEffectsSound())
            {
                main.soundEffect.SetupSoundEffect("sounds/badmove.wav");
                main.soundEffect.Play();
            }
            wrongMoves.Enqueue(1);
        }

        public void semiGoodMove()
        {
            if (main.gameSettings.getEffectsSound())
            {
                main.soundEffect.SetupSoundEffect("sounds/goodmove.wav");
                main.soundEffect.Play();
            }
        }

        public void goodMove()
        {
            if (main.gameSettings.getEffectsSound())
            {
                main.soundEffect.SetupSoundEffect("sounds/goodmove.wav");
                main.soundEffect.Play();
            }
            //points!!
            points += 5;
            score.Text = points + " pts";
            //add new figure
            int num = r.Next(1,4);
            for (int i = 0; i < num; i++)
            {
                monitor.addAnotherIcon();
            }
        }

        public void noMovesLeft()
        {
            monitor.stop();
            score.Foreground = new SolidColorBrush(Colors.Green);
            iconToFind.Text = "END!";
            iconToFind.Foreground = new SolidColorBrush(Colors.Green);
            keepCounting = false;

            main.restoreBackground();
            GameFinished gameFinished = new GameFinished(score.Text, main, this);
            body.Children.Clear();
            body.Children.Add(gameFinished);

            main.playSound.stopSound();
            if (main.gameSettings.getEffectsSound())
            {
                main.soundEffect.SetupSoundEffect("sounds/SadTrombone.wav");
                main.soundEffect.Play();
            }
        }

        
    }
}