﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using game.Model;

namespace game.Pages
{
    public partial class GameGrid : UserControl
    {

        int rows = 1, cols = 1;
        Dictionary<String, int> currentIcons = new Dictionary<String, int>();
        List<INotifier> listNotifiers = new List<INotifier>();

        public GameGrid()
        {
            InitializeComponent();
            InitializeGrid(1, 1);
        }


        public void InitializeGrid(int numRows, int numCols)
        {
            //clean all first
            currentIcons.Clear();
            mainGrid.ShowGridLines = true;
            mainGrid.Children.Clear();
            mainGrid.RowDefinitions.Clear();
            mainGrid.ColumnDefinitions.Clear();
            mainGrid.Visibility = Visibility.Visible;
            
            rows = numRows;
            cols = numCols;
            mainGrid.Children.Clear();

            for (int i = 0; i < numRows; i++)
            {
                RowDefinition rd = new RowDefinition();
                rd.Height = GridLength.Auto;
                mainGrid.RowDefinitions.Add(rd);
            }
            
            for (int j = 0; j < numCols; j++)
            {
                ColumnDefinition cd = new ColumnDefinition();
                cd.Width= GridLength.Auto;
                mainGrid.ColumnDefinitions.Add(cd);
            }
        }

        public void BlockGrid() {
            mainGrid.Visibility = Visibility.Collapsed;
        }

        public bool addIcon() {
            Random r = new Random(DateTime.Now.Millisecond);
            Coordinate c = new Coordinate(r.Next(0, rows - 1), r.Next(0, cols - 1));
            addIconCoordinate(c);
            return true;
        }


        public void addIcon2()
        {
            Random r = new Random(DateTime.Now.Millisecond);
            Coordinate c = new Coordinate(r.Next(0, rows - 1), r.Next(0, cols - 1));
            //star looking from a random position until the end of the cells
            for (int i = c.X() ; i < rows; i++)
            {
                for (int j = c.Y(); j < cols; j++)
                {
                    //check if it is free
                    if (isFree(i, j))
                    {
                        Coordinate c2 = new Coordinate(i, j);
                        addIconCoordinate(c2);
                        return;
                    }
                }
            }

            //star looking the begging until a random position
            for (int i = 0; i < c.X(); i++)
            {
                for (int j = 0; j < c.Y(); j++)
                {
                    //check if it is free
                    if (isFree(i, j))
                    {
                        Coordinate c2 = new Coordinate(i, j);
                        addIconCoordinate(c2);
                        return;
                    }
                }
            }
        }

         
        public bool addIcons(int num) {
            Random r = new Random(DateTime.Now.Millisecond);
            for (int i = 0; i < num; i++)
            {
                Coordinate c = new Coordinate(r.Next(0, rows), r.Next(0, cols));
                addIconCoordinate(c);
            }
            return true;
        }

        public bool fullIcons() {
            cleanAll();
            for (int i = 0; i < rows; i++)
            {
                for (int j = 0; j < cols; j++)
                {
                    Coordinate c = new Coordinate(i, j);
                    addIconCoordinate(c);
                }
            }
            return true;
        }

        public bool isFree(int numRow, int numCol) {
            for (int i = 0; i < mainGrid.Children.Count; i++)
            {
                UIElement child = mainGrid.Children[i];
                if (child == null)
                    continue;
                if ((int)child.GetValue(Grid.ColumnProperty) == numCol && (int)child.GetValue(Grid.RowProperty) == numRow)
                    return false;
            }
            return true;
        }


        public bool addIconPosition(int x, int y) {
            Coordinate c = new Coordinate(x, y);
            return addIconCoordinate(c);
        }

        public bool addIconCoordinate(Coordinate c) {
            //clean cell
            cleanPosition(c.X(), c.Y());

            Image img = ImageGenerator.getRandomImage();
            SpecialButton button; 
            
            if( mainGrid.RowDefinitions.Count > 5 )
                button = new SpecialButton(90, 90, img);
            else
                button = new SpecialButton(120, 120, img);

            button.button().Click += new RoutedEventHandler(buttonCliked);
            button.SetValue(Grid.RowProperty, c.X());
            button.SetValue(Grid.ColumnProperty, c.Y());


            mainGrid.Children.Add(button);

            if (!currentIcons.ContainsKey(button.getTag()))
                currentIcons.Add(button.getTag(), 1);
            else
                currentIcons[button.getTag()] = currentIcons[button.getTag()] + 1;

            return true;
        }

        void buttonCliked(object sender, RoutedEventArgs e)
        {
            Button buttonPressed = (Button)sender;
            notify((string)buttonPressed.Tag, sender);
        }


        public void removeIcon(Button button)
        {
            for (int i = 0; i < mainGrid.Children.Count; i++)
            {
                UIElement child = mainGrid.Children[i];
                if (child == null)
                    continue;
                SpecialButton s = (SpecialButton)child;
                if (s.button().Equals(button))
                {
                    //remove from keys
                    try
                    {
                        currentIcons[s.getTag()] = currentIcons[s.getTag()] - 1;
                        if (currentIcons[s.getTag()] == 0)
                            currentIcons.Remove(s.getTag());
                        //remove fromo grid
                        mainGrid.Children.Remove(child);
                    }
                    catch (Exception) { }
                    
                    return;
                }
            }
        }

        public void Disable(Button button)
        {
            button.IsEnabled = false;
            button.Opacity = 0.3;
        }

        public void EnableAll()
        {
            foreach (UIElement child in mainGrid.Children)
            {
                if (child == null)
                    continue;
                SpecialButton s = (SpecialButton)child;
                s.button().IsEnabled = true;
                s.button().Opacity = 1;
            }
        }

        public void DisableAll()
        {
            foreach (UIElement child in mainGrid.Children)
            {
                if (child == null)
                    continue;
                SpecialButton s = (SpecialButton)child;
                s.button().IsEnabled = false;
            }
        }

        public void cleanAll() {
            foreach (UIElement child in mainGrid.Children) {
                if (child == null)
                    continue;
                mainGrid.Children.Remove(child);
            }
            currentIcons.Clear();
        }

        public void cleanPosition(int numRow, int numCol){
            for (int i = 0; i< mainGrid.Children.Count; i++)
            {
                UIElement child = mainGrid.Children[i];
                if (child == null)
                    continue;
                if ((int)child.GetValue(Grid.ColumnProperty) == numCol && (int)child.GetValue(Grid.RowProperty) == numRow)
                {
                    SpecialButton s = (SpecialButton)child;
                    //remove from keys
                    currentIcons[s.getTag()] = currentIcons[s.getTag()] - 1;
                    if( currentIcons[s.getTag()] == 0 )
                        currentIcons.Remove(s.getTag());

                    mainGrid.Children.Remove(child);
                    return;
                }
            }
        }


        public void AddNotifier(INotifier notifier)
        {
            listNotifiers.Add(notifier);
        }


        private void notify(String message, Object sender) {
            foreach (INotifier noti in listNotifiers) {
                noti.notification(message, sender);
            }
        }

        public Dictionary<String, int> getIconsName()
        {
            return currentIcons;
        }
    }
}
